use actix_web::{dev::PeerAddr, get, http, web, Error, HttpRequest, HttpResponse};

use crate::{AppState, Claims};

#[get("/config")]
pub async fn get_config(req: HttpRequest, data: web::Data<AppState>) -> HttpResponse {
    let token = match req
        .headers()
        .get(http::header::AUTHORIZATION)
        .and_then(|h| h.to_str().ok())
        .filter(|s| s.starts_with("Bearer "))
        .map(|s| &s[7..])
    {
        Some(s) => s,
        None => return HttpResponse::Unauthorized().finish(),
    };

    let decoded =
        match jsonwebtoken::decode::<Claims>(token, &data.jwt_decoding_key, &data.jwt_validation) {
            Ok(x) => x,
            Err(_) => return HttpResponse::BadRequest().finish(),
        };

    if !data.device_ids.contains(&decoded.claims.sub) {
        return HttpResponse::Unauthorized().finish();
    }

    let buf = match data.devices_api_json.get(&decoded.claims.sub) {
        Some(x) => x,
        None => return HttpResponse::InternalServerError().finish(),
    };

    HttpResponse::Ok()
        .content_type("application/json")
        .body(buf.clone())
}

#[get("/image")]
pub async fn get_image(
    req: HttpRequest,
    peer_addr: Option<PeerAddr>,
    data: web::Data<AppState>,
    client: web::Data<reqwest::Client>,
) -> Result<HttpResponse, Error> {
    let token = match req
        .headers()
        .get(http::header::AUTHORIZATION)
        .and_then(|h| h.to_str().ok())
        .filter(|s| s.starts_with("Bearer "))
        .map(|s| &s[7..])
    {
        Some(s) => s,
        None => return Ok(HttpResponse::Unauthorized().finish()),
    };

    let decoded =
        match jsonwebtoken::decode::<Claims>(token, &data.jwt_decoding_key, &data.jwt_validation) {
            Ok(x) => x,
            Err(_) => return Ok(HttpResponse::BadRequest().finish()),
        };

    if !data.device_ids.contains(&decoded.claims.sub) {
        return Ok(HttpResponse::Unauthorized().finish());
    }

    let config = match data.devices.get(&decoded.claims.sub) {
        Some(x) => x,
        None => return Ok(HttpResponse::InternalServerError().finish()),
    };

    let mut forwarded_req = client.get(config.image.url.to_owned());
    forwarded_req = match peer_addr {
        Some(PeerAddr(addr)) => forwarded_req.header("x-forwarded-for", addr.ip().to_string()),
        None => forwarded_req,
    };

    let resp = forwarded_req
        .send()
        .await
        .map_err(actix_web::error::ErrorInternalServerError)?;

    let mut client_resp = HttpResponse::build(
        actix_web::http::StatusCode::from_u16(resp.status().as_u16())
            .map_err(actix_web::error::ErrorInternalServerError)?,
    );

    // Remove `Connection` as per
    // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Connection#Directives
    for (header_name, header_value) in resp.headers().iter().filter(|(h, _)| *h != "connection") {
        client_resp.insert_header((
            actix_web::http::header::HeaderName::from_bytes(header_name.as_ref()).unwrap(),
            actix_web::http::header::HeaderValue::from_bytes(header_value.as_ref()).unwrap(),
        ));
    }

    Ok(client_resp.streaming(resp.bytes_stream()))
}
